==> plasma_2.0.0.tar.gz: plasma_2.0.0/core_blas/core_ztslqt.f <==
      SUBROUTINE CORE_ZTSLQT( M, N, IB, A1, LDA1, A2, LDA2, T, LDT, 
     $                       TAU, WORK, INFO )

*********************************************************************
*     PLASMA core_blas routine (version 2.0.0)                      *
*     Release Date: July, 4th 2009                                  *
*     PLASMA is a software package provided by Univ. of Tennessee,  *
*     Univ. of California Berkeley and Univ. of Colorado Denver.    *
*********************************************************************
*
*     .. Scalar Arguments ..
      INTEGER            M, N, IB, LDA1, LDA2, LDT, INFO 
*     ..
*     .. Array Arguments ..
      COMPLEX*16            A1( LDA1, * ), A2( LDA2, * )
      COMPLEX*16            T( LDT, * ), TAU( * ), WORK( * ) 
*     ..
*
*  Purpose
*  =======
*
*  CORE_ZTSLQT computes a LQ factorization of a rectangular matrix
*  formed by coupling side-by-side a complex M-by-M 
*  lower triangular tile A1 and a complex M-by-N tile A2:
*  
*  | A1 A2 | = L * Q
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the tile A1 and A2.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the tile A2.  N >= 0.
*
*  IB      (input) INTEGER
*          The inner-blocking size.  IB >= 0.
*
*  A1      (input/output) COMPLEX*16 array, dimension (LDA1,M)
*          On entry, the M-by-M tile A1.
*          On exit, the elements on and below the diagonal of the array
*          contain the M-by-M lower trapezoidal tile L;
*          the elements above the diagonal are not referenced.
*
*  LDA1    (input) INTEGER
*          The leading dimension of the array A1.  LDA1 >= max(1,M).
*
*  A2      (input/output) COMPLEX*16 array, dimension (LDA2,N)
*          On entry, the M-by-N tile A2.
*          On exit, all the elements with the array TAU, represent 
*          the unitary tile Q as a product of elementary reflectors 
*          (see Further Details).
*
*  LDA2    (input) INTEGER
*          The leading dimension of the array A2.  LDA2 >= max(1,M).
*
*  T       (output) COMPLEX*16 array, dimension (LDT,N)
*          The IB-by-N triangular factor T of the block reflector.
*          T is upper triangular by block (economic storage);
*          The rest of the array is not referenced.
*
*  LDT     (input) INTEGER
*          The leading dimension of the array T. LDT >= IB.
*
*  TAU     (output) COMPLEX*16 array, dimension (min(M,N))
*          The scalar factors of the elementary reflectors (see Further
*          Details).
*
*  WORK    (workspace) COMPLEX*16 array, dimension (M)
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
*  Further Details
*  ===============
*
*  The tile Q is represented as a product of elementary reflectors
*
*     Q = H(k)' . . . H(2)' H(1)', where k = min(M,N).
*
*  Each H(i) has the form
*
*     H(i) = I - tau * v * v'
*
*  where tau is a complex scalar, and v is a complex vector with
*  v(1:i-1) = 0 and v(i) = 1; conjg(v(i+1:n)) is stored on exit in
*  A2(i,1:n), and tau in TAU(i).
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16            ZONE, ZZERO
      INTEGER            IONE
      PARAMETER          ( ZONE = ( 1.0D+0, 0.0D+0 ) )
      PARAMETER          ( ZZERO = ( 0.0D+0, 0.0D+0 ) )
      PARAMETER          ( IONE = 1 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, II, SB, IINFO
*     ..
*     .. External Subroutines ..
      EXTERNAL           ZLARFG, CORE_ZSSMLQ, XERBLA
      EXTERNAL           ZCOPY, ZGEMV, ZTRMV, ZAXPY, ZGERC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     Test the input arguments.
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( IB.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA2.LT.MAX( 1, M ) ) THEN
         INFO = -7
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'CORE_ZTSLQT', -INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( M.EQ.0 .OR. N.EQ.0 .OR. IB.EQ.0 )
     $   RETURN
*
      DO 10 II = 1, M, IB
         SB = MIN( M-II+1, IB )
         DO 20 I = 1, SB
*
*           Generate elementary reflector H( II*IB+I ) to annihilate
*           A( II*IB+I, II*IB+I:M ).
*
*#IFDEF COMPLEX .OR. COMPLEX_16
            CALL ZLACGV( N, A2( II+I-1, 1 ), LDA2 )
            CALL ZLACGV( 1, A1( II+I-1, II+I-1 ), LDA1 )
*#ENDIF
            CALL ZLARFG( N+1, A1( II+I-1, II+I-1 ), A2( II+I-1, 1 ),
     $                  LDA2, TAU( II+I-1 ) )

            IF( ( II+I-1 ).LT.M ) THEN
*
*              Apply H( II+I-1 ) to A( II+I:II+IB-1, II+I-1:M  ) from the right.
*
               CALL ZCOPY( SB-I, A1( II+I, II+I-1 ), IONE,
     $                    WORK, IONE)
               CALL ZGEMV( 'NoTranspose', SB-I, N, ZONE,
     $                    A2( II+I, 1 ), LDA2,
     $                    A2( II+I-1, 1 ), LDA2, ZONE, WORK, IONE )
               CALL ZAXPY( SB-I, -TAU( II+I-1 ), WORK, IONE,
     $                    A1( II+I, II+I-1 ), IONE )
               CALL ZGERC( SB-I, N, -TAU( II+I-1 ), WORK, IONE,
     $                   A2( II+I-1, 1 ), LDA2,
     $                   A2( II+I, 1 ),LDA2 )
            END IF
*
*           Calculate T.
*
            CALL ZGEMV( 'NoTranspose', I-1, N, -TAU( II+I-1 ),
     $                 A2( II, 1 ), LDA2, A2( II+I-1, 1 ), LDA2,
     $                 ZZERO, T( 1, II+I-1 ), IONE )
*#IFDEF COMPLEX .OR. COMPLEX_16
            CALL ZLACGV( N, A2( II+I-1, 1 ), LDA2 )
            CALL ZLACGV( 1, A1( II+I-1, II+I-1 ), LDA1 )
*#ENDIF
            CALL ZTRMV( 'Upper', 'NoTranspose', 'Nonunit', I-1,
     $                 T( 1, II ), LDT, T( 1, II+I-1 ), IONE )
            T( I, II+I-1 ) = TAU( II+I-1 )
 20      CONTINUE
         IF ( M.GT.( II+IB-1 ) ) THEN   
             CALL CORE_ZSSMLQ( 'Right', 'ConjTranspose',
     $                        SB, N, M-( II+SB-1 ), IB, IB,
     $                        A1( II+SB, II ), LDA1,
     $                        A2( II+SB, 1 ), LDA2,
     $                        A2( II, 1 ), LDA2, 
     $                        T( 1, II ), LDT, WORK, LDA1, IINFO)
         ENDIF
 10   CONTINUE
      RETURN
*
*     End of CORE_ZTSLQT.
*
      END

==> plasma_2.1.0.tar.gz: plasma_2.1.0/core_blas/core_ztslqt.f <==
      SUBROUTINE CORE_ZTSLQT( M, N, IB, A1, LDA1, A2, LDA2, T, LDT,
     $                       TAU, WORK, INFO )

*********************************************************************
*     PLASMA core_blas routine (version 2.1.0)                      *
*     Author: Hatem Ltaief                                          *
*     Release Date: November, 15th 2009                             *
*     PLASMA is a software package provided by Univ. of Tennessee,  *
*     Univ. of California Berkeley and Univ. of Colorado Denver.    *
*********************************************************************
*
*     .. Scalar Arguments ..
      INTEGER            M, N, IB, LDA1, LDA2, LDT, INFO 
*     ..
*     .. Array Arguments ..
      COMPLEX*16            A1( LDA1, * ), A2( LDA2, * )
      COMPLEX*16            T( LDT, * ), TAU( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  CORE_ZTSLQT computes a LQ factorization of a rectangular matrix
*  formed by coupling side-by-side a complex M-by-M
*  lower triangular tile A1 and a complex M-by-N tile A2:
*
*  | A1 A2 | = L * Q
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the tile A1 and A2.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the tile A2.  N >= 0.
*
*  IB      (input) INTEGER
*          The inner-blocking size.  IB >= 0.
*
*  A1      (input/output) COMPLEX*16 array, dimension (LDA1,M)
*          On entry, the M-by-M tile A1.
*          On exit, the elements on and below the diagonal of the array
*          contain the M-by-M lower trapezoidal tile L;
*          the elements above the diagonal are not referenced.
*
*  LDA1    (input) INTEGER
*          The leading dimension of the array A1.  LDA1 >= max(1,M).
*
*  A2      (input/output) COMPLEX*16 array, dimension (LDA2,N)
*          On entry, the M-by-N tile A2.
*          On exit, all the elements with the array TAU, represent 
*          the unitary tile Q as a product of elementary reflectors 
*          (see Further Details).
*
*  LDA2    (input) INTEGER
*          The leading dimension of the array A2.  LDA2 >= max(1,M).
*
*  T       (output) COMPLEX*16 array, dimension (LDT,N)
*          The IB-by-N triangular factor T of the block reflector.
*          T is upper triangular by block (economic storage);
*          The rest of the array is not referenced.
*
*  LDT     (input) INTEGER
*          The leading dimension of the array T. LDT >= IB.
*
*  TAU     (output) COMPLEX*16 array, dimension (min(M,N))
*          The scalar factors of the elementary reflectors (see Further
*          Details).
*
*  WORK    (workspace) COMPLEX*16 array, dimension (M)
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
*  Further Details
*  ===============
*
*  The tile Q is represented as a product of elementary reflectors
*
*     Q = H(k)' . . . H(2)' H(1)', where k = min(M,N).
*
*  Each H(i) has the form
*
*     H(i) = I - tau * v * v'
*
*  where tau is a complex scalar, and v is a complex vector with
*  v(1:i-1) = 0 and v(i) = 1; conjg(v(i+1:n)) is stored on exit in
*  A2(i,1:n), and tau in TAU(i).
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16            ZONE, ZZERO
      INTEGER            IONE
      PARAMETER          ( ZONE = ( 1.0D+0, 0.0D+0 ) )
      PARAMETER          ( ZZERO = ( 0.0D+0, 0.0D+0 ) )
      PARAMETER          ( IONE = 1 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, II, SB, IINFO
*     ..
*     .. External Subroutines ..
      EXTERNAL           ZLARFG, CORE_ZSSMLQ, XERBLA
      EXTERNAL           ZCOPY, ZGEMV, ZTRMV, ZAXPY, ZGERC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     Test the input arguments.
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( IB.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA2.LT.MAX( 1, M ) ) THEN
         INFO = -7
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'CORE_ZTSLQT', -INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( M.EQ.0 .OR. N.EQ.0 .OR. IB.EQ.0 )
     $   RETURN
*
      DO 10 II = 1, M, IB
         SB = MIN( M-II+1, IB )
         DO 20 I = 1, SB
*
*           Generate elementary reflector H( II*IB+I ) to annihilate
*           A( II*IB+I, II*IB+I:M ).
*
*#IFDEF COMPLEX .OR. COMPLEX_16
            CALL ZLACGV( N, A2( II+I-1, 1 ), LDA2 )
            CALL ZLACGV( 1, A1( II+I-1, II+I-1 ), LDA1 )
*#ENDIF
            CALL ZLARFG( N+1, A1( II+I-1, II+I-1 ), A2( II+I-1, 1 ),
     $                  LDA2, TAU( II+I-1 ) )

            IF( ( II+I-1 ).LT.M ) THEN
*
*              Apply H( II+I-1 ) to A( II+I:II+IB-1, II+I-1:M  ) from the right.
*
               CALL ZCOPY( SB-I, A1( II+I, II+I-1 ), IONE,
     $                    WORK, IONE)
               CALL ZGEMV( 'NoTranspose', SB-I, N, ZONE,
     $                    A2( II+I, 1 ), LDA2,
     $                    A2( II+I-1, 1 ), LDA2, ZONE, WORK, IONE )
               CALL ZAXPY( SB-I, -TAU( II+I-1 ), WORK, IONE,
     $                    A1( II+I, II+I-1 ), IONE )
               CALL ZGERC( SB-I, N, -TAU( II+I-1 ), WORK, IONE,
     $                   A2( II+I-1, 1 ), LDA2,
     $                   A2( II+I, 1 ),LDA2 )
            END IF
*
*           Calculate T.
*
            CALL ZGEMV( 'NoTranspose', I-1, N, -TAU( II+I-1 ),
     $                 A2( II, 1 ), LDA2, A2( II+I-1, 1 ), LDA2,
     $                 ZZERO, T( 1, II+I-1 ), IONE )
*#IFDEF COMPLEX .OR. COMPLEX_16
            CALL ZLACGV( N, A2( II+I-1, 1 ), LDA2 )
            CALL ZLACGV( 1, A1( II+I-1, II+I-1 ), LDA1 )
*#ENDIF
            CALL ZTRMV( 'Upper', 'NoTranspose', 'Nonunit', I-1,
     $                 T( 1, II ), LDT, T( 1, II+I-1 ), IONE )
            T( I, II+I-1 ) = TAU( II+I-1 )
 20      CONTINUE
         IF ( M.GT.( II+IB-1 ) ) THEN   
             CALL CORE_ZSSMLQ( 'Right', 'ConjTranspose',
     $                        SB, N, M-( II+SB-1 ), IB, IB,
     $                        A1( II+SB, II ), LDA1,
     $                        A2( II+SB, 1 ), LDA2,
     $                        A2( II, 1 ), LDA2, 
     $                        T( 1, II ), LDT, WORK, LDA1, IINFO)
         ENDIF
 10   CONTINUE
      RETURN
*
*     End of CORE_ZTSLQT.
*
      END
